package edu.georgetown.gucs.dictionary;

import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.Global;

/**
 * Prints statistics for a {@linkplain Dictionary}:
 * <ul>
 * <li>Name</li>
 * <li>Language</li>
 * <li>Number of documents</li>
 * <li>Number of tokens</li>
 * <li>If the dictionary has been trimmed (including the IDF range, if trimmed)</li>
 * <li>Minimum and maximum IDF</li>
 * <li>List of tokenizers</li>
 * </ul>
 * 
 * @author Clay Shields
 * 
 */
public class ShowDictionaryStatistics {

  private static Dictionary dictionary;

  /**
   * Prints the dictionary statistics for the specified dictionary
   * 
   * @param dictionary_file
   *          the string filename containing the serialized dictionary
   */
  public static void print(String dictionary_file) {
    dictionary = new Dictionary(dictionary_file);
    dictionary.showStatistics();
  }

  /**
   * Provides the dictionary statistics for the specified dictionary
   * 
   * @param dictionary_file
   *          the string filename containing the serialized dictionary
   * 
   * @return the string containing the specified dictionary's statistics
   */
  public static String getStatistics(String dictionary_file) {
    dictionary = new Dictionary(dictionary_file);
    return dictionary.getStatistics();
  }

  /**
   * Prints statistics for a dictionary, including name, language, number of documents, number of tokens, minimum and
   * maximum IDF, list of tokenizers, and whether the dictionary has been trimmed
   * 
   * @param args
   *          array of string command line arguments<br>
   *          <code>args[0]</code> the filename containing the dictionary to show
   */
  public static void main(String args[]) {
    try {
      ShowDictionaryStatistics.print(args[0]);
      Global.monitor.closeBuffer();
    } catch (ArrayIndexOutOfBoundsException e) {
      System.err.println("Usage: java ShowDictionaryStatstics <dictionary file name>");
      System.err.println("Prints statistics for a dictionary, including name, language, number of "
          + "documents, number of tokens, minimum and maximum IDF, list of tokenizers, and whether "
          + "the dictionary has been trimmed");
      System.exit(-1);
    } catch (RuntimeException exception) {
      ExceptionHandler.handleExceptionExit(exception);
    }
  }
}
