package edu.georgetown.gucs.tokenizers;

import java.util.Iterator;

import edu.georgetown.gucs.utility.Pair;

/**
 * Separates tokens based on punctuation and removes punctuation from tokens
 * 
 * @author Clay Shields
 * @author Lindsay Neubauer
 * 
 */
public class StripPunctuationTokenizer extends Tokenizer {

  private static final long serialVersionUID = -5556452429806285951L;
  // This is the posix format for punctuation characters
  private String delimiters = "\\p{Punct}";

  /**
   * Separates tokens based on punctuation and removes punctuation from tokens
   */
  @Override
  public void tokenize(Iterator<String> iterator) {
    synchronized (this.tokenVector) {
      this.tokenVector.clear();
      while (iterator.hasNext()) {
        String[] withoutPunctuation = iterator.next().split(this.delimiters, 0);
        for (String token : withoutPunctuation) {
          if (token.length() > 0) {
            this.tokenVector.add(token.trim());
          }
        }
      }
    }
  }

  /**
   * Separates tokens based on punctuation and removes punctuation from tokens
   */
  @Override
  public void tokenize(Iterator<String> tokensIterator, Iterator<Pair<Integer, Integer>> positionsIterator) {
    synchronized (this.tokenVector) {
      synchronized (this.positions) {
        this.tokenVector.clear();
        this.positions.clear();
        while (tokensIterator.hasNext() && positionsIterator.hasNext()) {
          Pair<Integer, Integer> pos = positionsIterator.next();
          String[] withoutPunctuation = tokensIterator.next().split(this.delimiters, 0);
          for (String token : withoutPunctuation) {            
            if (token.length() > 0) {
              this.tokenVector.add(token.trim());
              this.positions.add(pos);
            }
          }
        }
      }
    }
  }
}
