package sdtext;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Locale;

import javax.swing.ButtonGroup;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.LayoutStyle;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.jdom.Comment;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import edu.georgetown.gucs.experiment.Experiment;
import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.Global;

/**
 * Provides an interface for receiving action events related to an experiment; used with {@linkplain GUI}.
 * 
 * @author Lindsay Neubauer
 * 
 */
class ExperimentActionListener implements ActionListener {
  JButton xmlLink = new JButton();
  JButton datasetLink = new JButton();
  JButton plotsLink = new JButton();
  int numberOfTrialThreads = 0;
  int numberOfTrials = 3;
  boolean insert = false;
  String description = "";
  JComboBox languageChooser = new JComboBox();

  JRadioButton sampleCountRadioButton = new JRadioButton("File Count");
  JRadioButton samplePercentRadioButton = new JRadioButton("Percent Files");
  final JSpinner sampleSpinner = new JSpinner(new SpinnerNumberModel(10, 1, 100, 1));
  JRadioButton dictionaryCountRadioButton = new JRadioButton("File Count");
  JRadioButton dictionaryPercentRadioButton = new JRadioButton("Percent Files");
  final JSpinner dictionarySpinner = new JSpinner(new SpinnerNumberModel(10, 1, 100, 1));

  TokenizerSelectionHandler tokenizerSelectionHandler = new TokenizerSelectionHandler();
  DictionaryActionListener dictionaryActionListener = new DictionaryActionListener("create");
  ExperimentSelectionHandler fingerprinterSelectionHandler = new ExperimentSelectionHandler("FingerprinterEnum");
  ExperimentSelectionHandler matcherSelectionHandler = new ExperimentSelectionHandler("MatcherEnum");

  JButton experimentButton = new JButton("Run Experiment");
  JComponent[] plots;

  ExperimentActionListener() {
    GUI.linkSetup(this.xmlLink);
    GUI.linkSetup(this.datasetLink);
    GUI.linkSetup(this.plotsLink);
    this.plotsLink.setActionCommand("");

    this.plots = new JComponent[] { new JCheckBox("Precision vs. Recall"), new JCheckBox("Precision vs. Fscore"),
        new JCheckBox("Recall vs. Fscore"), new JCheckBox("Dictionary Size vs. Precision"),
        new JCheckBox("Dictionary Size vs. Recall"), new JCheckBox("Dictionary Size vs. Fscore"),
        new JCheckBox("Dictionary Size vs. Precision vs. Recall"), new JCheckBox("Mangler: Delete Tokens"),
        new JCheckBox("Mangler: Delete Characters"), new JCheckBox("Mangler: Delete Whitespace"),
        new JCheckBox("Mangler: Change Characters"), new JCheckBox("Mangler: Change Tokens"),
        new JCheckBox("Mangler: Split Tokens"), new JCheckBox("Mangler: Change to Punctuation"),
        new JCheckBox("Mangler: Delete Section"), new JCheckBox("Mangler: Delete Head"),
        new JCheckBox("Mangler: Delete Tail"), new JCheckBox("Mangler: Summary Reduce") };

    for (JComponent plot : this.plots) {
      String xmlPlotText = ((JCheckBox) plot).getText().toLowerCase(Locale.ENGLISH).replace("vs.", "_")
          .replace("size", "").replaceAll("\\s", "").replace("_averageprecision&averagerecall", "");
      if (!xmlPlotText.contains("precision") && !xmlPlotText.contains("recall") && !xmlPlotText.contains("fscore")) {
        xmlPlotText = "mangler " + xmlPlotText;
      }
      ((JCheckBox) plot).setActionCommand(xmlPlotText);
      if (((JCheckBox) plot).getActionCommand().equals("precision_recall")) {
        ((JCheckBox) plot).setSelected(true);
      }
    }
    this.experimentButton.setEnabled(false);
    this.experimentButton.addActionListener(this);
  }

  private boolean errorChecking() {
    // need dataset path
    if (this.datasetLink.getActionCommand().length() < 1) {
      this.experimentButton.setEnabled(false);
      JOptionPane.showConfirmDialog(this.experimentButton, "Must select a dataset directory to run an experiment",
          "Error: Experiment Dataset", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
      return false;
    }

    // need tokenizer(s)
    if (this.tokenizerSelectionHandler.getTokenizersAndManglers().size() < 1) {
      JOptionPane.showConfirmDialog(this.experimentButton,
          "Must add at least one set of tokenizers/manglers to run an experiment", "Error: Experiment Tokenizers",
          JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
      return false;
    }

    // need fingerprinter
    if (this.fingerprinterSelectionHandler.getSelected().size() < 1) {
      JOptionPane.showConfirmDialog(this.experimentButton,
          "Must select at least one fingerprinter to run an experiment", "Error: Experiment Fingerprinter",
          JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
      return false;
    }

    // need matcher
    if (this.matcherSelectionHandler.getSelected().size() < 1) {
      JOptionPane.showConfirmDialog(this.experimentButton, "Must select at least one matcher to run an experiment",
          "Error: Experiment Matcher", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
      return false;
    }

    // if no plots specified then at least save it with the dataset name
    if (this.plotsLink.getActionCommand().length() < 1) {
      this.plotsLink.setActionCommand(this.datasetLink.getActionCommand().substring(
          this.datasetLink.getActionCommand().lastIndexOf('/') + 1));
    }

    return true;
  }

  @Override
  public void actionPerformed(ActionEvent actionEvent) {
    // TODO: Add error checking (make sure at least one of everything is present)

    if (errorChecking()) {
      this.experimentButton.setEnabled(false);
      this.xmlLink.setText("Creating configuration file and running experiment...");
      JFileChooser fileChooser = new JFileChooser();
      fileChooser.setFileFilter(new FileNameExtensionFilter("XML Files", "xml"));
      fileChooser.setDialogTitle("Save as XML Experiment Configuration File");
      int returnVal = fileChooser.showSaveDialog(this.experimentButton);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        try {
          createExperimentConfigurationXML(fileChooser.getSelectedFile().getAbsolutePath());
          Experiment experiment = new Experiment(fileChooser.getSelectedFile().getAbsolutePath());
          experiment.showSettings();
          experiment.runExperiment();
          GUI.createLink(fileChooser.getSelectedFile().getAbsolutePath(), this.xmlLink, false);
        } catch (RuntimeException exception) {
          this.xmlLink.setText("");
          Global.monitor.printMessage("ExperimentActionListener", "Exception: " + exception.getMessage());
          exception.printStackTrace();

          Global.monitor.writeToBuffer("ExperimentActionListener", "experimentButton.actionPerformed",
              exception.getMessage());
          JOptionPane.showConfirmDialog((Component) actionEvent.getSource(), exception.getMessage(),
              "Error: Experiment", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
        }
      } else {
        this.xmlLink.setText("");
      }
      this.experimentButton.setEnabled(true);
    }
  }

  private void createExperimentConfigurationXML(String filename) {
    Global.monitor.writeToBuffer("ExperimentActionListener", "createExperimentConfigurationXML", "start");

    // Setup experiment root
    Element experiment = new Element("experiment");
    if (this.numberOfTrialThreads < 1) {
      this.numberOfTrialThreads = Runtime.getRuntime().availableProcessors();
    }
    if (this.numberOfTrials < 3) {
      this.numberOfTrials = 3;
    }
    experiment.setAttribute("threads", Integer.toString(this.numberOfTrialThreads));
    experiment.setAttribute("trials", Integer.toString(this.numberOfTrials));
    experiment.setAttribute("DB_insert", Boolean.toString(this.insert));
    experiment.setAttribute("description", this.description);

    // Setup plots (if present)
    Element plotsElement = new Element("plots");
    plotsElement.setAttribute("path", this.plotsLink.getActionCommand());
    boolean plotSelected = false;
    for (JComponent plot : this.plots) {
      if (((JCheckBox) plot).isSelected()) {
        plotSelected = true;
        Element plotElement = new Element("plot");
        String[] plotNameSplit = ((JCheckBox) plot).getActionCommand().split("\\s");
        plotElement.setAttribute("name", plotNameSplit[0]);
        if (plotNameSplit.length > 1) {
          plotElement.setAttribute("parameter", plotNameSplit[1]);
        }
        plotsElement.addContent(plotElement);
      }
    }
    if (plotSelected) {
      experiment.addContent(new Comment("Creates different scattered plots displaying results from this experiment"));
      experiment.addContent(plotsElement);
    }

    // Setup dataset
    experiment.addContent(new Comment("name = the name of the dataset being used; "
        + "path = The full path to the dataset"));
    Element datasetElement = new Element("dataset");
    datasetElement.setAttribute("name",
        this.datasetLink.getActionCommand().substring(this.datasetLink.getActionCommand().lastIndexOf('/') + 1));
    datasetElement.setAttribute("path", this.datasetLink.getActionCommand());

    datasetElement.addContent(new Comment(String.format("%n%s%n%s%n%s%n%s%n%s%n%s",
        "      Running over all files in a directory can take a very long time, so",
        "      it is possible to specify a sample size. This is either a percent of the",
        "      files not used for dictionary creation, or a flat count of files not already",
        "      used. If the count is larger than all remaining files, all will be used (More",
        "      than 10,000 files can take a long time to run with many different edits.).",
        "      You can specify many of these at a time, and it will iterate over each.")));
    Element sampleElement = new Element("sample");
    if (this.samplePercentRadioButton.isSelected()) {
      sampleElement.setAttribute("percent", Integer.toString((Integer) this.sampleSpinner.getValue()));
    } else {
      sampleElement.setAttribute("count", Integer.toString((Integer) this.sampleSpinner.getValue()));
    }
    datasetElement.addContent(sampleElement);

    Element dictionaryElement = new Element("dictionary");
    dictionaryElement.setAttribute("language", (String) this.languageChooser.getSelectedItem());
    Element createDictElement = new Element("create");
    if (this.dictionaryPercentRadioButton.isSelected()) {
      createDictElement.setAttribute("percent", Integer.toString((Integer) this.dictionarySpinner.getValue()));
    } else {
      createDictElement.setAttribute("count", Integer.toString((Integer) this.dictionarySpinner.getValue()));
    }
    dictionaryElement.addContent(new Comment(String.format("%n%s%n%s%n%s%n%s%n%s",
        "        This specifies the number of files to be used in creating a dictionary.",
        "        In general, this should be low, say under 25%, so there are files left for",
        "        testing against. As in the sample, the sample size can be specified by either",
        "        by percent or by file count. You can specify many of these, and all will", "        be run.")));
    dictionaryElement.addContent(createDictElement);

    dictionaryElement.addContent(new Comment(String.format("%n%s%n%s%n%s%n%s%n%s%n%s%n%s%n%s%n%s%n%s%n%s",
        "        The inverse document frequency (IDF) is a measure of how often a",
        "        particular term appeared in the set of documents examined. Terms with a low",
        "        IDF are generally too common to be useful in differentiating documents, while",
        "        ones with a high IDF are too specific to be very useful. We deal with a normalized",
        "        IDF. Each IDF is divided by the largest IDF seen in the set, and thus range",
        "        from 0 to 1. It is best to trim the dictionary, removing the low IDF terms",
        "        and the high ones. One of the primary purposes of this program is to conduct",
        "        the search to determine that range. 0.0 (min) & 1.0 (max) are used if nothing",
        "        listed, otherwise it tries each combination of min and max IDF. Ranges where",
        "        the minIDF is greater or equal to the maxIDF are ignored. A good place to",
        "        start is minIDF between .3 and .5 and maxIDF between .6 and .8.")));
    for (String idfRange : this.dictionaryActionListener.getIDFValues()) {
      Element idfElement = new Element("IDF");
      idfElement.setAttribute("min", String.valueOf(Double.valueOf(idfRange.split(",")[0]) / 100));
      idfElement.setAttribute("max", String.valueOf(Double.valueOf(idfRange.split(",")[1]) / 100));
      dictionaryElement.addContent(idfElement);
    }
    datasetElement.addContent(dictionaryElement);
    experiment.addContent(datasetElement);

    // Setup fingerprinter
    // TODO: ! matcher cosineSimilarity gives different scores depending on dictionary inside
    // --> check to see if true when dictionary exactly the same

    // TOOD: ! Multiple cosine similarity parameters??
    experiment.addContent(new Comment("There are different methods to create fingerprints."));
    for (String fingerprinter : this.fingerprinterSelectionHandler.getSelected()) {
      Element fingerprinterElement = new Element("fingerprinter");
      fingerprinterElement.setAttribute("name", fingerprinter);
      experiment.addContent(fingerprinterElement);
    }

    // Setup matcher
    experiment.addContent(new Comment("There are different methods of matching. "
        + "Scores for boolean matching are scaled as 0 or 99."));
    for (String matcher : this.matcherSelectionHandler.getSelected()) {
      Element matcherElement = new Element("matcher");
      String[] splitMatcher = matcher.split("\\s");
      matcherElement.setAttribute("name", splitMatcher[0]);
      if (splitMatcher.length > 1) {
        matcherElement.setAttribute("score", splitMatcher[1]);
      }
      experiment.addContent(matcherElement);
    }

    // Setup tokenizers (including FileManglerTokenizer)
    for (String tokenizers : this.tokenizerSelectionHandler.getTokenizersAndManglers()) {
      boolean addedManglers = false;
      String manglers = tokenizers.substring(tokenizers.indexOf("Mangler(s):")).replace("Mangler(s):", "").trim();
      tokenizers = tokenizers.substring(0, tokenizers.indexOf("Mangler(s):")).replace("Tokenizer(s): ", "").trim();

      // Add tokenizers
      Element tokenizerGroupElement = new Element("tokenizers");
      for (String tokenizer : tokenizers.split(",")) {
        String[] tokenizerSplit = tokenizer.trim().split("\\s");
        Element tokenizerElement = new Element("tokenizer");
        tokenizerElement.setAttribute("name", tokenizerSplit[0]);
        if (tokenizerSplit.length > 1) {
          tokenizerElement.setAttribute("parameter", tokenizerSplit[1]);
        }
        tokenizerGroupElement.addContent(tokenizerElement);

        // If there are manglers to add, do it after the first (file) tokenizer
        if (manglers.length() > 0 && !addedManglers) {
          addedManglers = true;
          Element fileManglerElement = new Element("tokenizer");
          fileManglerElement.setAttribute("name", "FileManglerTokenizer");
          fileManglerElement
              .addContent(new Comment(
                  String
                      .format("%n%s%n%s",
                          "        Files are manipulated in a variety of ways to test robustness to edits. The code ",
                          "        that performs edits is called the mangler. Each mangler can only be used once per set of manglers.")));
          Element manglerGroupElement = new Element("manglers");
          for (String mangler : manglers.split(",")) {
            mangler = mangler.replace("Mangler(s):", "").trim();
            String[] manglerSplit = mangler.split("\\s");
            Element manglerElement = new Element("mangler");
            manglerElement.setAttribute("name", manglerSplit[0]);
            if (manglerSplit.length > 1) {
              manglerElement.setAttribute("percent", manglerSplit[1]);
            }
            manglerGroupElement.addContent(manglerElement);
          }
          fileManglerElement.addContent(manglerGroupElement);
          tokenizerGroupElement.addContent(fileManglerElement);
        }
      }
      experiment.addContent(tokenizerGroupElement);
    }

    try {
      Document doc = new Document();
      doc.addContent(new Comment(String.format("%n%s%n%s%n%s%n%s%n%s%n%s%n%s%n%s%n%s%n",
          "  This is the configuration file for the Experiment program. This program",
          "  creates a number of dictionaries using the variety of parameter settings",
          "  provided. For each dictionary, it fingerprints many files, then compares",
          "  those fingerprints against automatically edited versions of the same file",
          "  and outputs the results. The results may also be inserted into a database.",
          "  In general, we expect that the settings that provide the most robust results",
          "  in this process will work best when applied to other files in the directory",
          "  provided in the dataset parameter. This program can take a long time to run",
          "  as the settings chosen affect the search space, which can be large.")));
      doc.addContent(new Comment(String.format("%n%s%n%s%n%s%n%n%s%n%s%n%n%s%n%s%n%s%n%s%n%s%n%s%n%n%s",
          "  This code is threaded. The threads parameter below sets the number of",
          "  threads to use to run trials. By default it will use one thread per core.",
          "  To turn off threading set threads=\"1\".",
          "  The number of trials tells how many runs to do for each parameter setting.",
          "  Three (3) is the minimum; 5-10 is better to ensure that results are accurate.",
          "  The DB_insert parameter specifies whether to store the results of this experiment",
          "  in the database associated with the given dataset If true, you must create the",
          "  database for the dataset by running", "  > psql # create database <dataset name>",
          "  Then go into the database and load trunk/doc/database_layout. The username and password",
          "  in DBInterface.java needs to match the user and password of the created database.",
          "  The description provides a human readable descriptor to be added to the database.")));
      doc.addContent(experiment);
      FileOutputStream dictionaryOut = new FileOutputStream(filename);
      Format form = Format.getPrettyFormat();
      form.setEncoding("UTF-8");
      XMLOutputter out = new XMLOutputter(form);
      out.output(doc, dictionaryOut);
      Global.monitor.printMessage("ExperimentActionListener", "Writing experiment configuration file to " + filename);
      dictionaryOut.close();
    } catch (FileNotFoundException e) {
      ExceptionHandler.handleExceptionExit(e);
    } catch (IOException e) {
      ExceptionHandler.handleExceptionExit(e);
    }

    Global.monitor.writeToBuffer("ExperimentActionListener", "createExperimentConfigurationXML", "end");
  }

  /**
   * Provides the panel for the selection of tokenizer experiment variables
   * 
   * @return the JPanel containing the tokenizer components for experiment selection
   */
  protected JPanel createTokenizerPanel() {
    JPanel tokenizersPanel = new JPanel();
    this.tokenizerSelectionHandler.createPanel(GUI.createGroupLayout(tokenizersPanel), true, true);
    return tokenizersPanel;
  }

  /**
   * Provides the panel for the selection of fingerprinter experiment variables
   * 
   * @return the JPanel containing the fingerprinter components for experiment selection
   */
  protected JPanel createFingerprintPanel() {
    JPanel fingerprinterPanel = new JPanel();
    this.fingerprinterSelectionHandler.createPanel(GUI.createGroupLayout(fingerprinterPanel));
    return fingerprinterPanel;
  }

  /**
   * Provides the panel for the selection of matcher experiment variables
   * 
   * @return the JPanel containing the matcher components for experiment selection
   */
  protected JPanel createMatcherPanel() {
    JPanel matcherPanel = new JPanel();
    this.matcherSelectionHandler.createPanel(GUI.createGroupLayout(matcherPanel));
    return matcherPanel;
  }

  /**
   * Provides the panel for the selection of dataset experiment variables
   * 
   * @return the JPanel containing the dataset components for experiment selection
   */
  protected JPanel createDatasetPanel() {
    JPanel directoryPanel = GUI.createLoadPanel("Dataset Directory", JFileChooser.DIRECTORIES_ONLY, this.datasetLink,
        this.experimentButton, Global.ALL_FILES);

    this.languageChooser.addItem("Arabic");
    this.languageChooser.addItem("Chinese");
    this.languageChooser.addItem("English");
    this.languageChooser.setSelectedItem("English");
    JLabel languageLabel = new JLabel("Dataset Language");
    languageLabel.setFont(Global.boldUnifontFont);

    JLabel idfLabel = new JLabel("Normalized IDF Range");
    idfLabel.setFont(Global.boldUnifontFont);
    JPanel idfPanel = this.dictionaryActionListener.createIDFPanel(idfLabel, true);

    // Sample and dictionary
    JPanel sampleDictionaryPanel = getDatasetFilesPanel();
    JPanel languagePanel = new JPanel(new GridLayout(1, 2));
    languagePanel.add(languageLabel);
    languagePanel.add(this.languageChooser);
    JPanel datasetPanel = new JPanel();
    GroupLayout layout = GUI.createGroupLayout(datasetPanel);
    layout.setHorizontalGroup(layout.createParallelGroup().addComponent(directoryPanel).addComponent(languagePanel)
        .addComponent(sampleDictionaryPanel).addComponent(idfPanel));
    layout.setVerticalGroup(layout.createSequentialGroup().addComponent(directoryPanel)
        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(languagePanel)
        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(sampleDictionaryPanel)
        .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(idfPanel));
    return datasetPanel;
  }

  private JPanel getDatasetFilesPanel() {
    this.dictionaryPercentRadioButton.setSelected(true);
    this.dictionaryCountRadioButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent action) {
        ExperimentActionListener.this.dictionarySpinner.setModel(new SpinnerNumberModel(100, 1, 5000, 1));
      }
    });
    this.dictionaryPercentRadioButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent action) {
        ExperimentActionListener.this.dictionarySpinner.setModel(new SpinnerNumberModel(20, 1, 100, 1));
      }
    });
    JLabel dictionaryLabel = new JLabel("Dictionary Parameter");
    dictionaryLabel.setFont(Global.boldUnifontFont);
    ButtonGroup dictionaryButtonGroup = new ButtonGroup();
    dictionaryButtonGroup.add(this.dictionaryCountRadioButton);
    dictionaryButtonGroup.add(this.dictionaryPercentRadioButton);
    JPanel dictionaryRadioPanel = new JPanel(new GridLayout(1, 2));
    dictionaryRadioPanel.add(this.dictionaryCountRadioButton);
    dictionaryRadioPanel.add(this.dictionarySpinner);
    JPanel dictionarySpinnerPanel = new JPanel();
    GroupLayout dictionaryLayout = GUI.createGroupLayout(dictionarySpinnerPanel);
    dictionaryLayout.setHorizontalGroup(dictionaryLayout.createParallelGroup().addComponent(dictionaryLabel)
        .addComponent(this.dictionaryPercentRadioButton).addComponent(dictionaryRadioPanel));
    dictionaryLayout.setVerticalGroup(dictionaryLayout.createSequentialGroup().addComponent(dictionaryLabel)
        .addComponent(this.dictionaryPercentRadioButton).addComponent(dictionaryRadioPanel));

    this.samplePercentRadioButton.setSelected(true);
    this.sampleCountRadioButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent action) {
        ExperimentActionListener.this.sampleSpinner.setModel(new SpinnerNumberModel(100, 1, 5000, 1));
      }
    });
    this.samplePercentRadioButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent action) {
        ExperimentActionListener.this.sampleSpinner.setModel(new SpinnerNumberModel(40, 1, 100, 1));
      }
    });
    JLabel sampleLabel = new JLabel("Sample Parameter");
    sampleLabel.setFont(Global.boldUnifontFont);
    ButtonGroup sampleButtonGroup = new ButtonGroup();
    sampleButtonGroup.add(this.sampleCountRadioButton);
    sampleButtonGroup.add(this.samplePercentRadioButton);
    JPanel sampleRadioPanel = new JPanel(new GridLayout(1, 2));
    sampleRadioPanel.add(this.sampleCountRadioButton);
    sampleRadioPanel.add(this.sampleSpinner);
    JPanel sampleSpinnerPanel = new JPanel();
    GroupLayout sampleLayout = GUI.createGroupLayout(sampleSpinnerPanel);
    sampleLayout.setHorizontalGroup(sampleLayout.createParallelGroup().addComponent(sampleLabel)
        .addComponent(this.samplePercentRadioButton).addComponent(sampleRadioPanel));
    sampleLayout.setVerticalGroup(sampleLayout.createSequentialGroup().addComponent(sampleLabel)
        .addComponent(this.samplePercentRadioButton).addComponent(sampleRadioPanel));

    JPanel datasetPanel = new JPanel();
    GroupLayout layout = GUI.createGroupLayout(datasetPanel);
    layout.setHorizontalGroup(layout.createParallelGroup().addComponent(dictionarySpinnerPanel)
        .addComponent(sampleSpinnerPanel));
    layout.setVerticalGroup(layout.createSequentialGroup().addComponent(dictionarySpinnerPanel)
        .addComponent(sampleSpinnerPanel));
    layout.linkSize(SwingConstants.VERTICAL, sampleSpinnerPanel, dictionarySpinnerPanel);

    return datasetPanel;
  }

  /**
   * Provides the panel for the selection of plots experiment variables
   * 
   * @return the JPanel containing the plots components for experiment selection
   */
  protected JPanel createPlotsPanel() {
    JPanel plotsPanel = new JPanel();
    plotsPanel.setMaximumSize(new Dimension(5000, 500));
    GroupLayout layout = GUI.createGroupLayout(plotsPanel);
    JPanel firstPlots = GUI.createSubPanel(Arrays.asList(this.plots).subList(0, 3), "Relevance Scores Plots", false);
    JPanel secondPlots = GUI.createSubPanel(Arrays.asList(this.plots).subList(3, 7),
        "Dictionary Size vs. Relevance Scores Plots", false);
    JPanel thirdPlots = GUI.createSubPanel(Arrays.asList(this.plots).subList(7, 17),
        "Mangler vs. Relevance Scores Plots", false);
    JPanel plotsDirectory = GUI.createLoadPanel("Plots Directory", JFileChooser.DIRECTORIES_ONLY, this.plotsLink, null,
        Global.ALL_FILES);
    layout.setHorizontalGroup(layout
        .createParallelGroup()
        .addComponent(plotsDirectory)
        .addGroup(
            layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup().addComponent(firstPlots).addComponent(secondPlots))
                .addComponent(thirdPlots)));
    layout.setVerticalGroup(layout
        .createSequentialGroup()
        .addComponent(plotsDirectory)
        .addGroup(
            layout.createParallelGroup()
                .addGroup(layout.createSequentialGroup().addComponent(firstPlots).addComponent(secondPlots))
                .addComponent(thirdPlots)));
    return plotsPanel;
  }

  /**
   * Provides the panel for saving and running an experiment
   * 
   * @return the JPanel containing the experiment components
   */
  protected JPanel createButtonPanel() {
    JPanel experimentPanel = new JPanel();
    experimentPanel.setMaximumSize(new Dimension(5000, 100));
    GroupLayout layout = GUI.createGroupLayout(experimentPanel);
    layout.setHorizontalGroup(layout.createParallelGroup().addComponent(this.xmlLink)
        .addComponent(this.experimentButton));
    layout.setVerticalGroup(layout.createSequentialGroup().addComponent(this.xmlLink)
        .addComponent(this.experimentButton));
    return experimentPanel;
  }
}
